@@ -1,10 +1,12 @@
 from torchvision.datasets import MNIST, CIFAR10, CIFAR100, ImageFolder
+from .ImageFiles import ImageFiles
 
 DATASETS = {
     "MNIST": MNIST,
     "CIFAR10": CIFAR10,
     "CIFAR100": CIFAR100,
-    "ImageFolder": ImageFolder
+    "ImageFolder": ImageFolder,
+    "ImageFiles": ImageFiles
 }
 
 
