@@ -11,9 +11,13 @@ from .utils import *
 
 
 class Module(nn.Module):
-    def __init__(self, epochs=20, batch_size=32, device=None, output_dir="output", pretrained_file=None, num_device=None):
+    def __init__(self, task_name=None, epochs=20, batch_size=32, device=None, output_dir="output", pretrained_file=None, num_device=None):
         super(Module, self).__init__()
 
+        self.task_name = task_name
+        if task_name is not None:
+            output_dir = os.path.join(output_dir, str(task_name))
+
         if device is None:
             device = "cuda" if torch.cuda.is_available() else "cpu"
             if device == "cuda":
@@ -66,7 +70,7 @@ class Module(nn.Module):
 
             score, is_best = None, False
             if do_val:
-                score = self.val_epoch(val_loader)
+                score = self.val_epoch(epoch, val_loader)
                 if score is not None:
                     if self.best_direction == "high":
                         if score >= self.best_score:
@@ -88,7 +92,10 @@ class Module(nn.Module):
         t, c = Timer(), Counter()
         t.start()
         for step, inputs in enumerate(train_loader):
-            inputs = [x.to(self.device) if isinstance(x, torch.Tensor) else x for x in inputs]
+            if isinstance(inputs, torch.Tensor):
+                inputs = (inputs.to(self.device),)
+            else:
+                inputs = [x.to(self.device) if isinstance(x, torch.Tensor) else x for x in inputs]
             reader_time = t.elapsed_time()
 
             metrics = self.train_step(*inputs)
@@ -132,17 +139,20 @@ class Module(nn.Module):
         if hasattr(self, "optimizer") and update_optimizer:
             self.optimizer.step()
 
-    def val_epoch(self, val_loader) -> Union[float, None]:
+    def val_epoch(self, epoch, val_loader) -> Union[float, None]:
         if val_loader is None:
             return
 
         self.train(False)
 
         c = Counter()
-        for step, data in enumerate(val_loader):
-            data = [x.to(self.device) if isinstance(x, torch.Tensor) else x for x in data]
+        for step, inputs in enumerate(val_loader):
+            if isinstance(inputs, torch.Tensor):
+                inputs = (inputs.to(self.device),)
+            else:
+                inputs = [x.to(self.device) if isinstance(x, torch.Tensor) else x for x in inputs]
 
-            score, metrics = self.forward(*data)
+            score, metrics = self.forward(*inputs)
 
             for k in metrics:
                 if isinstance(metrics[k], torch.Tensor) and metrics[k].ndim == 0:
@@ -157,7 +167,7 @@ class Module(nn.Module):
 
         msg = "".join(f"{k}={c.get(k):.4f} " for k in c.data.keys())
         self.log(f"[{datetime.datetime.now():%Y-%m-%d %H:%M:%S}] [VAL] {msg}")
-        self.log_tb({k: c.get(k) for k in c.data}, "val")
+        self.log_tb({k: c.get(k) for k in c.data}, "val", step=epoch)
 
         print()
         return c.score
@@ -222,14 +232,18 @@ class Module(nn.Module):
         if to_file and self.log_file is not None:
             print(msg, end='\n', flush=True, file=open(self.log_file, "a+"))
 
-    def log_tb(self, infos: Dict, mode: str):
+    def log_tb(self, infos: Dict, mode: str, step=None):
+        if step is None:
+            step = self.global_step
+
         for k, v in infos.items():
             if isinstance(v, float):
-                self.logger.add_scalar(f"{mode}/{k}", v, global_step=self.global_step)
+                self.logger.add_scalar(f"{mode}/{k}", v, global_step=step)
             elif isinstance(v, torch.Tensor) and v.dim == 4 and int(v.shape[1]) in [1, 3, 4]:
-                self.logger.add_images(f"{mode}/{k}", v, global_step=self.global_step)
+                self.logger.add_images(f"{mode}/{k}", v, global_step=step)
             elif isinstance(v, torch.Tensor) and v.dim == 3 and int(v.shape[0]) in [1, 3, 4]:
-                self.logger.add_image(f"{mode}/{k}", v, global_step=self.global_step)
+                self.logger.add_image(f"{mode}/{k}", v, global_step=step)
+
         self.logger.flush()
 
 
